/* ****************************************************************************
 * Copyright (c) 2002 Java Eclipse Extension Project.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.opensource.org/licenses/cpl.php
 * ============================================================================
 * $Header: /cvsroot/jeextension/jp.sourceforge.jeextension.styleeditor/src/jp/sourceforge/jeextension/styleeditor/core/StyleEditorTools.java,v 1.8 2007/10/25 03:16:49 kohnosuke Exp $
 * $Revision: 1.8 $
 * $Date: 2007/10/25 03:16:49 $
 * ============================================================================
 * ***************************************************************************/
package jp.sourceforge.jeextension.styleeditor.core;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import jp.sourceforge.jeextension.common.xml.XMLNode;
import jp.sourceforge.jeextension.styleeditor.StyleEditorPlugin;
import jp.sourceforge.jeextension.styleeditor.entry.DefinedStyleEntry;
import jp.sourceforge.jeextension.styleeditor.entry.DependsEntry;
import jp.sourceforge.jeextension.styleeditor.entry.DependsEntryList;
import jp.sourceforge.jeextension.styleeditor.entry.FontEntry;
import jp.sourceforge.jeextension.styleeditor.entry.IDependsEntry;
import jp.sourceforge.jeextension.styleeditor.entry.IEntry;
import jp.sourceforge.jeextension.styleeditor.entry.IPartitionEntry;
import jp.sourceforge.jeextension.styleeditor.entry.PartitionEntry;
import jp.sourceforge.jeextension.styleeditor.entry.PartitionEntryList;
import jp.sourceforge.jeextension.styleeditor.text.ExtEndOfLineRule;
import jp.sourceforge.jeextension.styleeditor.text.ExtMultiLineRule;
import jp.sourceforge.jeextension.styleeditor.text.ExtRegExpRule;
import jp.sourceforge.jeextension.styleeditor.text.ExtSingleLineRule;
import jp.sourceforge.jeextension.styleeditor.text.ExtTextAttribute;
import jp.sourceforge.jeextension.styleeditor.text.ExtWordRule;
import jp.sourceforge.jeextension.styleeditor.text.IColorActivator;
import jp.sourceforge.jeextension.styleeditor.text.IExtRule;

import org.eclipse.jdt.ui.text.IColorManager;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IDocumentPartitioner;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.DefaultPartitioner;
import org.eclipse.jface.text.rules.IPredicateRule;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.RuleBasedPartitionScanner;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.text.rules.WordRule;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.RGB;

/**
 * StyleEditorTools
 * @since 1.0.0
 * <!-- #IntID:00001 Refactoring -->
 * <!-- #IntID:00004 Style Maximization -->
 * <!-- #IntID:00006 Compatibility previous version conf -->
 * <!-- #ReqID:10017 RegExp -->
 * <!-- #IntID:00010 // singleline comment type defined -->
 */
public class StyleEditorTools {
    
    /**
     * Checking RegExp values that '\r' '\n' is not contain.
     * @param regexp value
     * @return true when checking ok.
     */
    public static String checkRegExp(String regexp) {
        
        StringBuffer detected = new StringBuffer();
        if (regexp.indexOf("\\r") != -1) {
            detected.append("\\r");
        }
        if (regexp.indexOf("\\n") != -1) {
            detected.append("\\n");
        }
        if (regexp.indexOf("\\x0D") != -1) {
            detected.append("\\x0D");
        }
        if (regexp.indexOf("\\x0A") != -1) {
            detected.append("\\x0A");
        }
        if (regexp.indexOf("\\x0d") != -1) {
            detected.append("\\x0d");
        }
        if (regexp.indexOf("\\x0a") != -1) {
            detected.append("\\x0a");
        }
        
        if (detected.length() != 0) {
            return detected.toString();
        }
        else {
            return null;
        }
    }
    
    /**
     * Change IDependsEntry with XMLNode.
     * @param entry IDependsEntry
     * @param node  XMLNode
     * @param ext   extension name. (ex. java, means *.java)
     */
    public static void changeDependsEntry(IDependsEntry entry, XMLNode node,
            String ext) {
        DependsEntry newEntry = new DependsEntry(node, ext);
        entry.setDepends(newEntry.getDepends());
        entry.setRGB(newEntry.getRGB());
        entry.setRule(newEntry.getRule());
        entry.setWords(newEntry.getWords());
        entry.setBgColor(newEntry.isBgColor());
        entry.setBgRGB(newEntry.getBgRGB());
        entry.setItalic(newEntry.isItalic());
        entry.setBold(newEntry.isBold());
        entry.setStrikeThrough(newEntry.isStrikeThrough());
        entry.setUnderline(newEntry.isUnderline());
    }
    
    /**
     * Change IPartitionEntry with XMLNode.
     * @param entry IPartitionEntry
     * @param node  XMLNode
     * @param ext   extension name. (ex. java, means *.java)
     */
    public static void changePartitionEntry(IPartitionEntry entry,
            XMLNode node, String ext) {
        IPartitionEntry newEntry = new PartitionEntry(node, ext);
        entry.setRGB(newEntry.getRGB());
        entry.setRule(newEntry.getRule());
        entry.setStart(newEntry.getStart());
        entry.setEnd(newEntry.getEnd());
        entry.setName(newEntry.getName());
        entry.setRegExp(newEntry.getRegExp());
        entry.setView(newEntry.getView());
        entry.setBgColor(newEntry.isBgColor());
        entry.setBgRGB(newEntry.getBgRGB());
        entry.setItalic(newEntry.isItalic());
        entry.setBold(newEntry.isBold());
        entry.setStrikeThrough(newEntry.isStrikeThrough());
        entry.setUnderline(newEntry.isUnderline());
    }
    
    /**
     * Create IExtRule form XMLNode.
     * @param node XMLNode.
     * @param ext   extension name. (ex. java, means *.java)
     * @return IExtRule
     */
    public static IExtRule createExtRule(XMLNode node, String ext) {
        IExtRule rule = null;
        String nodeName = node.getNodeName();
        RGB rgb = new RGB(0, 0, 0);
        rgb.red   = Integer.parseInt(node.getAttribute("red"));
        rgb.green = Integer.parseInt(node.getAttribute("green"));
        rgb.blue  = Integer.parseInt(node.getAttribute("blue"));
        RGB brgb = null;
        int style = SWT.NORMAL;
        Object obj = node.getAttribute("bgcolor");
        if (obj != null) {
            if ("true".equals(node.getAttribute("bgcolor"))) {
                brgb = new RGB(0, 0, 0);
                brgb.red   = Integer.parseInt(node.getAttribute("bred"));
                brgb.green = Integer.parseInt(node.getAttribute("bgreen"));
                brgb.blue  = Integer.parseInt(node.getAttribute("bblue"));
            }
            if ("true".equals(node.getAttribute("bold"))) {
                style = SWT.BOLD;
            }
            else {
                style = SWT.NORMAL;
            }
            if ("true".equals(node.getAttribute("italic"))) {
                style |= SWT.ITALIC;
            }
            if ("true".equals(node.getAttribute("strikethrough"))) {
                style |= 1 << 29;
// #IntID:00008 Eclipse 3.0.2 Support, TextAttribute Constants
//                style |= TextAttribute.STRIKETHROUGH;
            }
            if ("true".equals(node.getAttribute("underline"))) {
                style |= 1 << 30;
// #IntID:00008 Eclipse 3.0.2 Support, TextAttribute Constants
//                style |= TextAttribute.UNDERLINE;
            }
        }
        
        if ("partition".equals(nodeName)) {
            String start = node.getAttribute("start");
            String end   = node.getAttribute("end");
            String regexp = node.getAttribute("regexp");
            IToken token  = new Token(node.getAttribute("name"));
            if (regexp != null && regexp.equals("") == false) {
                rule = new ExtRegExpRule(node.getAttribute("regexp"), token);
            } else 
            // #IntID:00010 // singleline comment type defined
            if ("".equals(end) == true) {
                rule = new ExtEndOfLineRule(start, token);
            }
            else {
                rule = new ExtMultiLineRule(start, end, token);
            }
        }
        if ("multiLine".equals(nodeName)) {
            String start = node.getAttribute("start");
            String end   = node.getAttribute("end");
            IToken token = new Token(new ExtTextAttribute(rgb, brgb, style));
            rule = new ExtMultiLineRule(start, end, token);
        }
        if ("singleLine".equals(nodeName)) {
            IToken token = new Token(new ExtTextAttribute(rgb, brgb, style));
            String start = node.getAttribute("start");
            String end   = node.getAttribute("end");
            rule = new ExtSingleLineRule(start, end, token);
        }
        if ("endOfLine".equals(nodeName)) {
            IToken token = new Token(new ExtTextAttribute(rgb, brgb, style));
            rule = new ExtEndOfLineRule(node.getAttribute("start"), token);
        }
        if ("keyword".equals(nodeName)) {
            IToken token = new Token(new ExtTextAttribute(rgb, brgb, style));
            ExtWordRule wRule = new ExtWordRule(new StyleEditorWordDetector());
            String word = node.getAttribute("word");
            String csensitive = node.getAttribute("case");
            if (csensitive == null) {
                csensitive = "true";
            }
            wRule.addWord(word, token, csensitive);
            rule = wRule;
        }
        if ("regexp".equals(nodeName)) {
            IToken token = new Token(new ExtTextAttribute(rgb, brgb, style));
            rule = new ExtRegExpRule(node.getAttribute("regexp"), token);
        }
        return rule;
    }
    
    /**
     * Shortcut method for getting PartitionEntryMap.
     * @return PartitionEntryMap.
     */
    public static Map getPartitionEntryMap() {
        DefinedStyleEntry styleEntry =
                StyleEditorPlugin.getDefinedStyleEntry();
        return styleEntry.getPartitionEntryMap();
    }
    
    /**
     * Shortcut method for getting FontEntryMap.
     * @return  FontEntryMap
     */
    public static Map getFontEntryMap() {
        DefinedStyleEntry styleEntry =
                StyleEditorPlugin.getDefinedStyleEntry();
        return styleEntry.getFontEntryMap();
    }
    
    /**
     * Return Editor Font.
     * @param ext   file's extension.
     * @return  editor font.
     */
    public static Font getFont(String ext) {
        Map fontEntryMap = getFontEntryMap();
        FontEntry fontEntry = (FontEntry) fontEntryMap.get(ext);
        if (fontEntry != null) {
            return fontEntry.getFont();
        }
        else {
            fontEntry = (FontEntry) fontEntryMap.get("*");
            if (fontEntry != null) {
                return fontEntry.getFont();
            }
            else {
                return JFaceResources.getTextFont();
            }
        }
    }
    
    /**
     * Shortcut method for getting IPartitionEntry.
     * @param partName depends partition name.
     * @param ext   extension name. (ex. java, means *.java)
     * @param create if not exist, create one.
     * @return IPartitionEntry.
     */
    public static IPartitionEntry getPartitionEntry(String partName,
            String ext, boolean create) {
        DefinedStyleEntry styleEntry =
                StyleEditorPlugin.getDefinedStyleEntry();
        return styleEntry.getPartitionEntry(partName, ext, create);
    }
    
    /**
     * Shortcut method for getting PartitionEntryList.
     * @param   ext extension name. (ex. java, means *.java)
     * @param create if not exist, create one.
     * @return PartitionEntryList.
     */
    public static PartitionEntryList getPartitionEntryList(String ext,
            boolean create) {
        DefinedStyleEntry styleEntry =
                StyleEditorPlugin.getDefinedStyleEntry();
        return styleEntry.getPartitionEntryList(ext, create);
    }
    
    /**
     * Initiate Tools.
     */
    StyleEditorTools() {
        super();
    }
    
    /**
     * Create Partition Scanner.
     * @param ext       extension name. (ex. java, means *.java)
     * @return RuleBasedPartitionScanner.
     */
    public RuleBasedPartitionScanner createPartitionScanner(String ext) {
        RuleBasedPartitionScanner scanner = new RuleBasedPartitionScanner();
        scanner.setPredicateRules(createPredicateRule(ext));
        return scanner;
    }
    
    /**
     * Create BasicScanner for default content type.
     * @param partitionName target's partition name.
     * @param ext   extension name. (ex. java, means *.java)
     * @return RuleBasedScanner.
     */
    public RuleBasedScanner createScanner(String partitionName, String ext) {
        RuleBasedScanner scanner = new RuleBasedScanner();
        scanner.setRules(createRules(partitionName, ext));
        return scanner;
    }
    
    /**
     * Create ContentTypes includes IDocument.DEFAULT_CONTENT_TYPE.
     * @param ext       extension name. (ex. java, means *.java)
     * @return String[] contains ContentTypes.
     */
    public static String[] getContentTypes(String ext) {
        List typeList = new ArrayList();
        typeList.add(IDocument.DEFAULT_CONTENT_TYPE);
        return createContentTypes(typeList, ext);
    }
    
    /**
     * Create ContentTypes exclude IDocument.DEFAULT_CONTENT_TYPE.
     * @param ext       extension name. (ex. java, means *.java)
     * @return String[] contains ContentTypes.
     */
    public static String[] getContentTypesExcludeDefault(String ext) {
        return createContentTypes(new ArrayList(), ext);
    }
    
    /**
     * Return default token using depends partition entry.
     * @param entry partition entry.
     * @return default token.
     */
    public IToken getDefaultToken(IPartitionEntry entry) {
        IColorManager cm =
            StyleEditorPlugin.getDefault().getColorManager();
        RGB rgb = entry.getRGB();
        RGB brgb = null;
        Color fg = cm.getColor(rgb);
        Color bg = null;
        if (entry.isBgColor()) {
            brgb = entry.getBgRGB();
            bg = cm.getColor(brgb);
        }
        int style = SWT.NORMAL;
        if (entry.isBold()) {
            style = SWT.BOLD;
        }
        else {
            style = SWT.NORMAL;
        }
        if (entry.isItalic()) {
            style |= SWT.ITALIC;
        }
        if (entry.isStrikeThrough()) {
            style |= 1 << 29;
// #IntID:00008 Eclipse 3.0.2 Support, TextAttribute Constants
//            style |= TextAttribute.STRIKETHROUGH;
        }
        if (entry.isUnderline()) {
            style |= 1 << 30;
// #IntID:00008 Eclipse 3.0.2 Support, TextAttribute Constants
//            style |= TextAttribute.UNDERLINE;
        }
        
        TextAttribute attr = new TextAttribute(fg, bg, style);
        IToken token = new Token(attr);
        return token;
    }
    
    /**
     * Create a IDocumentPartitioner.
     * @param ext   extension name. (ex. java, means *.java)
     * @return IDocumentPartitioner instance.
     */
    public IDocumentPartitioner createDocumentPartitioner(String ext) {
        RuleBasedPartitionScanner scanner = createPartitionScanner(ext);
        String[] contentTypes =
            StyleEditorTools.getContentTypesExcludeDefault(ext);
        
        return new DefaultPartitioner(scanner, contentTypes);
    }
    /**
     * Activate color divice in rule for current editor.
     * @param ruleList List contains IExtRules.
     */
    private void activateColor(List ruleList) {
        IColorManager cm =
            StyleEditorPlugin.getDefault().getColorManager();
        for (int i = 0; i < ruleList.size(); i++) {
            Object obj = ruleList.get(i);
            if (obj instanceof IColorActivator) {
                IColorActivator rule = (IColorActivator) obj;
                rule.activateColor(cm);
            }
        }
    }
    
    /**
     * Append to ruleList.
     * @param entry     IEntry that defined entry.
     * @param ruleList  append's target.
     */
    private void appendRuleList(IEntry entry, List ruleList) {
        IRule rule = null;
        if (entry.getName().equals("keyword")) {
            // search WordRule
            for (int i = 0; i < ruleList.size(); i++) {
                Object obj = ruleList.get(i);
                if (obj instanceof WordRule) {
                    rule = (WordRule) obj;
                }
            }
            if (rule != null) {
                ExtWordRule wRule = (ExtWordRule) entry.getRule();
                ((ExtWordRule) rule).addWordRuleMap(wRule.getWordRuleMap());
                return;
            }
            else {
                rule = entry.getRule();
                
                // once clear all.
                ExtWordRule wRule = (ExtWordRule) entry.getRule();
                Map currentMap = new HashMap(wRule.getWordRuleMap());
                wRule.getWordRuleMap().clear();
                
                // append current entry.
                wRule.getWordRuleMap().putAll(currentMap);
            }
        }
        if (entry.getName().equals("multiLine")) {
            rule = entry.getRule();
        }
        if (entry.getName().equals("endOfLine")) {
            rule = entry.getRule();
        }
        if (entry.getName().equals("singleLine")) {
            rule = entry.getRule();
        }
        if (entry.getName().equals("regexp")) {
            rule = entry.getRule();
        }
        if (entry instanceof IPartitionEntry) {
            if (entry.getName().equals(IDocument.DEFAULT_CONTENT_TYPE)) {
                return;
            }
            rule = entry.getRule();
        }
        ruleList.add(rule);
    }
    
    /**
     * Create ContentTypes using List.
     * @param typeList  List contains ContentTypes.
     * @param ext       extension name. (ex. java, means *.java)
     * @return String[] contains ContentTypes.
     */
    private static String[] createContentTypes(List typeList, String ext) {
        PartitionEntryList entryList = getPartitionEntryList(ext, false);
        for (int i = 0; i < entryList.size(); i++) {
            IPartitionEntry entry = entryList.getPartitionEntry(i);
            String partition = entry.getName();
            if (!partition.equals(IDocument.DEFAULT_CONTENT_TYPE)) {
                typeList.add(entry.getName());
            }
        }
//        if (ext.equals("java")) {
//            if (!typeList.contains(IPartitionEntry.JAVA_DOC)) {
//                typeList.add(IPartitionEntry.JAVA_DOC);
//            }
//            if (!typeList.contains(IPartitionEntry.JAVA_MULTILINE)) {
//                typeList.add(IPartitionEntry.JAVA_MULTILINE);
//            }
//            typeList.add(IPartitionEntry.JAVA_SINGLELINE);
//            typeList.add(IPartitionEntry.JAVA_STRING);
//            typeList.add(IPartitionEntry.JAVA_CHAR);
//        }
        String[] types = new String[typeList.size()];
        return (String[]) typeList.toArray(types);
    }
    
    /**
     * Create IPredicateRule for Parition Scanner.
     * @param ext       extension name. (ex. java, means *.java)
     * @return created IPredicateRules.
     */
    private IPredicateRule[] createPredicateRule(String ext) {
        PartitionEntryList entryList = getPartitionEntryList(ext, false);
        
        List ruleList = new ArrayList();
        for (int i = 0; i < entryList.size(); i++) {
            IPartitionEntry entry = entryList.getPartitionEntry(i);
            appendRuleList(entry, ruleList);
        }
        activateColor(ruleList);
        
        IPredicateRule[] rules = new IPredicateRule[ruleList.size()];
        return (IPredicateRule[]) ruleList.toArray(rules);
    }
    
    /**
     * Create IRule for Basic Scanner.
     * @param partName target partition name.
     * @param ext   extension name. (ex. java, means *.java)
     * @return created IRules.
     */
    private IRule[] createRules(String partName, String ext) {
        IPartitionEntry partList = getPartitionEntry(partName, ext, false);
        DependsEntryList entryList = partList.getDependsEntryList();
        
        List ruleList = new ArrayList();
        for (int i = 0; i < entryList.size(); i++) {
            IDependsEntry entry = entryList.getDependsEntry(i);
            String partition = entry.getDepends().getName();
            if (partition.equals(partName)) {
                appendRuleList(entry, ruleList);
            }
        }
        activateColor(ruleList);
        
        IRule[] rules = new IRule[ruleList.size()];
        ruleList.toArray(rules);
        return rules;
    }
}
